/*
   program tftest

   Test the turnfast, turnpar, turnpro and turnrates routines.
*/

#include <math.h>
 
#include "SpiceUsr.h"
#include "SpiceZmc.h"
#include "SpiceZfc.h"

#define ROOM 1000

int main ()
{

   #define ROOM 1000
 
   SpiceBoolean found;

   SpiceChar    sym       [] = { 'x', 'y', 'z' }; 
   SpiceChar    symout;
   
   SpiceDouble  angacc    [3];
   SpiceDouble  angacc2   [ROOM][3];
   SpiceDouble  angvel    [ROOM][3];
   SpiceDouble  alpha     [3];
   SpiceDouble  conacc    [3];
   SpiceDouble  convel    [3];
   SpiceDouble  dir       ;
   SpiceDouble  maxacc    [3];
   SpiceDouble  maxvel    [3];
   SpiceDouble  omega     [3];
   SpiceDouble  savacc    [3];
   SpiceDouble  savvel    [3];
   SpiceDouble  t;
   SpiceDouble  ta        [ROOM];
   SpiceDouble  tb;
   SpiceDouble  tc;
   SpiceDouble  totacc;
   SpiceDouble  totang;
   SpiceDouble  trnrot    [ROOM][3][3];
   SpiceDouble  trntim    [ROOM];
   SpiceDouble  turn_angle;
   SpiceDouble  turn_axis [3];
   SpiceDouble  tv        [ROOM];
   SpiceDouble  tang;
   SpiceDouble  tvel;

   SpiceInt     i;
   SpiceInt     ier;
   SpiceInt     j;
   SpiceInt     k;
   SpiceInt     na;
   SpiceInt     nb;
   SpiceInt     nc;
   SpiceInt     nv;
   SpiceInt     nout;

/*
   Get program inputs.
*/

   printf( "Enter angular velocity constraint vector: " );
   scanf( "%lf %lf %lf", &convel[0], &convel[1], &convel[2] );
   printf( "convel = %f %f %f\n\n", convel[0], convel[1], convel[2] );

   printf( "Enter angular acceleration constraint vector: " );
   scanf( "%lf %lf %lf", &conacc[0], &conacc[1], &conacc[2] );
   printf( "conacc = %f %f %f\n\n", conacc[0], conacc[1], conacc[2] );

   printf( "Enter turn axis vector: " );
   scanf( "%lf %lf %lf", &turn_axis[0], &turn_axis[1], &turn_axis[2] );
   printf( "turn axis vector = %f %f %f\n\n", 
            turn_axis[0], turn_axis[1], turn_axis[2] );

   printf( "Enter turn axis angle: " );
   scanf( "%lf", &turn_angle );
   printf( "turn_angle = %f\n\n", turn_angle );

   printf( "Enter number of points in boost phase:");
   scanf( "%d", &nb );
   printf( "number of boost phase points = %d \n", nb );

   printf( "Enter number of points in coast phase:");
   scanf( "%d", &nc );
   printf( "number of coast phase points = %d \n", nc );

/*
   Test turnfast.
*/


   turnfast ( convel,
              conacc,
              turn_axis,
              turn_angle,
             &t,
             &tb,
             &tc,
              alpha,
              omega,
             &found,
             &ier         );

    
   if ( found != SPICETRUE ) { 

      printf ( "turnfast: no turn solution, ier = %d \n", ier );
      exit(1); 

   } else { 

      printf( "\n" ); 
      printf( "turnfast results:\n");
      printf( "-------------------\n");
      printf( "turn vector = %f %f %f \n", 
               turn_axis[0], turn_axis[1], turn_axis[2]);
      printf( "turn angle   = %f \n", turn_angle);
      printf( "total turn duration = %f seconds\n", t );
      printf( "spinup duration = %f seconds\n", tb );
      printf( "coast duration = %f seconds\n", tc );
      printf( "angular velocity components = %e %e %e rad/sec\n", 
              omega[0], omega[1], omega[2]);
      printf( "angular acceleration components = %e %e %e rad/sec^2\n",
              angacc[0], angacc[1], angacc[2]);
    }

/*

/*
   Test turnfast's output with turnpar.
*/

   totacc = vnorm_c ( alpha );

   turnpar ( turn_axis, 
             turn_angle, 
             t,
             totacc,
            &tb,
            &tc, 
             alpha,
             omega,
            &found,
            &ier    );
 

   if ( found != SPICETRUE ) { 

      printf ( "turnpar: no turn solution, ier = %d \n", ier );
      exit(1); 

   } else { 

      printf( "\n" ); 
      printf( "turnpar results (check of turnfast):\n" );
      printf( "--------------------------------------\n" );
      printf( "turn axis vector = %f %f %f \n", 
               turn_axis[0], turn_axis[1], turn_axis[2]);
      printf( "turn_angle   = %f \n", turn_angle);
      printf( "total angular acceleration = %f rad/sec^2\n", totacc);
      printf( "total turn duration = %f seconds\n", t );
      printf( "spinup duration = %f seconds\n", tb );
      printf( "coast duration = %f seconds\n", tc );
      printf( "angular velocity components = %e %e %e rad/sec\n",
               omega[0], omega[1], omega[2] );
      printf( "angular acceleration components = %e %e %e rad/sec^2\n",
               alpha[0], alpha[1], alpha[2] );
    }

/*
   Test turnpro
*/

   turnpro ( t, tb, tc, alpha, nb, nc, ROOM,        
             &nout, trntim, trnrot );

   printf( "\n" );
   printf( "turnpro results \n" );
   printf( "--------------------\n");
   printf( "number of boost points (nb) = %d \n", nb );
   printf( "number of coast points (nc)  = %d\n", nc );
   printf( "total number of points = 2*nb + nc = %d\n\n", nout );

/*
   printf( "orientation times and rates = \n" );

   for( i=0; i<nout; i++ ) { 
      printf( "\n" );
      printf( "%lf \n", trntim[i] );
      for( j=0; j<3; j++ ) {
         printf( "%f %f %f \n", trnrot[i][j][0], 
                                trnrot[i][j][1],  
                                trnrot[i][j][2] ); 
      } 
   }

*/

/*
   Recover angular velocities and accelerations numerically. 
*/

   turnrates ( ROOM, nout, trntim, trnrot, 
               &nv, tv,  angvel,  
               &na, ta,  angacc2 );

   printf( "\n" );
   printf( "turnrates results:\n" );
   printf( "--------------------\n" );
   printf( "total number of input points = %d \n", nout );
   printf( "total number of output ang. velocity vectors  = %d \n", nv );
   printf( "total number of output ang. acc. vectors    = %d \n", na );

/*
   Output angular velocity vectors.
*/

   printf ( "\n" );
   printf ( "time and angular velocity vectors:\n" );
   printf ( "-----------------------------------\n" );
   for ( i=0; i<nv; i++ ) { 
      printf( "%f %e %e %e \n", 
              tv[i], angvel[i][0], angvel[i][1], angvel[i][2] );
   }

/*
   Output angular acceleration vectors.
*/

   printf ( "\n" );
   printf ( "time and angular acceleration vectors:\n" );
   printf ( "----------------------------------------\n" );
   for ( i=0; i<na; i++ ) { 
      printf( "%f %e %e %e \n", 
              ta[i], angacc2[i][0], angacc2[i][1], angacc2[i][2] );
   }


/*
   Recover total angle by integrating angular velocity.
*/

   totang = 0.0;
   
   for ( i=0; i<nv; i++ ) { 
      if ( i < (nv-1) ) {
         totang = totang + (tv[i+1]-tv[i])*vnorm_c ( angvel[i]); 
      }
   }

   printf("\n");
   printf("-------------------------------------------------------------\n");
   printf("turn angle by integrating angular velocity = %e rad\n", totang);
   printf("-------------------------------------------------------------\n");

/*
   Determine maximum values for constaint checks.
*/

   for ( i=1; i<3; i++ ) { 
      maxvel[i] = 0.0;
      maxacc[i] = 0.0;
   }

   for ( i=0; i<nv; i++ ) { 
      for ( j=0; j<3; j++ ) {
         if ( fabs(angvel[i][j]) > maxvel[j] ) { 
             maxvel[j] = fabs(angvel[i][j]);
             savvel[j] = angvel[i][j];
         }
      }
   }

   for ( i=0; i<na; i++ ) { 
      for ( j=0; j<3; j++ ) {
         if ( fabs(angacc2[i][j]) > maxacc[j] ) { 
             maxacc[j] = fabs(angacc2[i][j]);
             savacc[j] = angacc2[i][j];
         }
      }
   }


/*
   Compare computed rates with constraints.
*/ 

   printf( "\n" );
   printf( "--------------------------------------------\n" );
   printf( "     Constraint Checks              \n" ); 
   printf( "--------------------------------------------\n" );
   printf( "  angular velocity                          \n" );
   printf( "--------------------------------------------\n" );
   printf( "axis      value      constraint \n" );
   printf( "--------------------------------------------\n" ); 
  
   for ( i=0; i<3; i++ ) { 
      symout = ( fabs(savvel[i]) <= convel[i] ) ? '<' : '>'; 
      printf( "%c  fabs(%e)  %c  %e\n", sym[i], savvel[i], symout, convel[i] ); 
   }
   
   
   printf( "------------------------------------------------\n" );
   printf( "  angular acceleration \n" );
   printf( "------------------------------------------------\n" ); 
   printf( "axis      value      constraint \n" );
   printf( "------------------------------------------------\n" ); 
  
   for ( i=0; i<3; i++ ) { 
      symout = ( fabs(savacc[i]) <= conacc[i] ) ? '<' : '>'; 
      printf( "%c  fabs(%e)  %c  %e\n", sym[i], savacc[i], symout, conacc[i] ); 
   }

   printf( "\n" ); 
   
   exit(0);

}
